@@ -32,6 +32,9 @@ struct
        k v
      | #ceff as e ->
        `Let ((local x ty, e), k (local x ty))
+     | `CAMLparam (xs, c) ->
+	let Ty t = Type_C.ccomp c in
+	`CAMLparam (xs, (c, t) >>= k)
      | `LetConst (y, i, c) ->
        (* let x = (let const y = i in c) in e
           ~>
@@ -74,6 +77,11 @@ struct
                           (value @-> returning (ptr void)),
                    [x])
 
+  let bytes_to_ptr : cexp -> ccomp =
+    fun x -> `App (reader "CTYPES_PTR_OF_OCAML_BYTES"
+                          (value @-> returning (ptr void)),
+                   [x])
+
   let float_array_to_ptr : cexp -> ccomp =
     fun x -> `App (reader "CTYPES_PTR_OF_FLOAT_ARRAY"
                           (value @-> returning (ptr void)),
@@ -85,10 +93,10 @@ struct
                    [x])
 
   let acquire_runtime_system : ccomp =
-    `App (conser "caml_acquire_runtime_system" (ptr void @-> returning void), [])
+    `App (conser "caml_acquire_runtime_system" (void @-> returning void), [])
 
   let release_runtime_system : ccomp =
-    `App (conser "caml_release_runtime_system" (ptr void @-> returning void), [])
+    `App (conser "caml_release_runtime_system" (void @-> returning void), [])
 
   let val_unit : ceff = `Global { name = "Val_unit";
                                   references_ocaml_heap = true;
@@ -140,7 +148,7 @@ struct
     | Array _ -> report_unpassable "arrays"
     | Bigarray _ -> report_unpassable "bigarrays"
     | OCaml String -> Some (string_to_ptr x)
-    | OCaml Bytes -> Some (string_to_ptr x)
+    | OCaml Bytes -> Some (bytes_to_ptr x)
     | OCaml FloatArray -> Some (float_array_to_ptr x)
 
   let prj ty x = prj ty ~orig:ty x
@@ -174,9 +182,9 @@ struct
   let fundec : type a. string -> a Ctypes.fn -> cfundec =
     fun name fn -> `Fundec (name, args fn, return_type fn)
 
-  let fn : type a. errno:errno_policy ->
+  let fn : type a. concurrency:[`Sequential|`Unlocked] -> errno:errno_policy ->
     cname:string -> stub_name:string -> a Ctypes_static.fn -> cfundef =
-    fun ~errno:errno_ ~cname ~stub_name f ->
+    fun ~concurrency ~errno:errno_ ~cname ~stub_name f ->
       let fvar = { fname = cname;
                    allocates = false;
                    reads_ocaml_heap = false;
@@ -186,14 +194,29 @@ struct
          | Returns t ->
            let x = fresh_var () in
            let e = `App (fvar, (List.rev vars :> cexp list)) in
-           begin match errno_ with
-               `Ignore_errno -> `Let ((local x t, e), (inj t (local x t) :> ccomp))
-             | `Return_errno -> 
+           begin match errno_, concurrency with
+               `Ignore_errno, `Sequential -> `Let ((local x t, e), (inj t (local x t) :> ccomp))
+             | `Ignore_errno, `Unlocked ->
+                release_runtime_system >>
+                `Let ((local x t, e), 
+                      acquire_runtime_system >>
+                     (((inj t (local x t) :> ccomp), value) >>= fun x ->
+		      `CAMLreturnT (Ty value, x) :> ccomp))
+             | `Return_errno, `Sequential -> 
                (`LetAssign (errno,
                            `Int Signed.SInt.zero,
                            `Let ((local x t, e),
                                  ((inj t (local x t) :> ccomp), value) >>= fun v ->
                                  (pair_with_errno v :> ccomp))) : ccomp)
+             | `Return_errno, `Unlocked -> 
+               (`LetAssign (errno,
+                           `Int Signed.SInt.zero,
+                           release_runtime_system >>
+                           `Let ((local x t, e),
+                                 (acquire_runtime_system >>
+                                 (inj t (local x t) :> ccomp), value) >>= fun v ->
+                                 ((pair_with_errno v :> ccomp), value) >>= fun x ->
+				 `CAMLreturnT (Ty value, x))) : ccomp)
            end
          | Function (x, f, t) ->
            begin match prj f (local x value) with
@@ -204,9 +227,12 @@ struct
            end
       in
       let f' = name_params f in
-      `Function (`Fundec (stub_name, value_params f', Ty value),
-                 body [] f',
-                `Extern)
+      let vp = value_params f' in
+      `Function (`Fundec (stub_name, vp, Ty value),
+		 (match concurrency with
+		   `Unlocked -> `CAMLparam(List.map fst vp, body [] f')
+		 | `Sequential -> body [] f'),
+                 `Extern)
 
   let byte_fn : type a. string -> a Ctypes_static.fn -> int -> cfundef =
     fun fname fn nargs ->
@@ -297,9 +323,9 @@ struct
 
 end
 
-let fn ~errno ~cname ~stub_name fmt fn =
+let fn ~concurrency ~errno ~cname ~stub_name fmt fn =
   let `Function (`Fundec (f, xs, _), _, _) as dec
-      = Generate_C.fn ~errno ~stub_name ~cname fn
+      = Generate_C.fn ~concurrency ~errno ~stub_name ~cname fn
   in
   let nargs = List.length xs in
   if nargs > max_byte_args then begin
@@ -346,11 +372,14 @@ struct
   let structure_type stub_name = 
     structure (sprintf "job_%s" stub_name)
 
-  let structure ~errno ~stub_name fmt fn args result =
+  let structure (type r) ~errno ~stub_name fmt fn args (result : r typ) =
     let open Ctypes in
     let s = structure_type stub_name in
-    let (_ : (_,_) field) = field s "job" lwt_unix_job in
-    let (_ : (_,_) field) = field s "result" result in
+    let _ : (_,_) field = field s "job" lwt_unix_job in
+    let () = match result with
+        Void -> let _ : (_,_) field = field s "result" int in ()
+      | result -> let _ : (_,_) field = field s "result" result in ()
+    in
     let () = match errno with
         `Ignore_errno -> ()
       | `Return_errno -> ignore (field s "error_status" sint)
@@ -360,7 +389,7 @@ struct
     let () = seal s in
     fprintf fmt "@[%a@];@\n" (fun t -> format_typ t) s
     
-  let worker ~errno ~cname ~stub_name fmt f result args =
+  let worker (type r) ~errno ~cname ~stub_name fmt f (result : r typ) args =
     let fn' = { fname = cname;
                allocates = false;
                reads_ocaml_heap = false;
@@ -368,21 +397,24 @@ struct
     and j = "j", Ty (ptr (structure_type stub_name)) in
     let rec body args : _ -> ccomp = function
         [] ->
-        let r c =
-          Generate_C.cast ~from:(Ty result) ~into:(Ty Void)
-            (`LetAssign (`PointerField (`Local j, "result"),
-                         `App (fn', List.rev args),
-                         c))
+        let r c = match result with
+          | Void -> Generate_C.(`App (fn', List.rev args) >> c)
+          | result ->
+            Generate_C.cast ~from:(Ty result) ~into:(Ty Void)
+              (`LetAssign (`PointerField (`Local j, "result"),
+                           `App (fn', List.rev args),
+                           c))
         in
         begin match errno with
             `Ignore_errno -> r (`Return (Ty Void, (`Int Signed.SInt.zero)))
           | `Return_errno ->
             let open Generate_C in
-              r
-              (`LetAssign
-                 (`PointerField (`Local j, "error_status"),
-                  errno,
-                  `Return (Ty Void, (`Int Signed.SInt.zero))))
+            `LetAssign (errno, `Int Signed.SInt.zero,
+                        r
+                          (`LetAssign
+                             (`PointerField (`Local j, "error_status"),
+                              errno,
+                              `Return (Ty Void, (`Int Signed.SInt.zero)))))
         end
       | (BoxedType ty, x) :: xs ->
         Generate_C.((`DerefField (`Local j, x), ty) >>= fun y ->
@@ -393,7 +425,7 @@ struct
                   body [] args,
                   `Static))
 
-  let result ~errno ~stub_name fmt fn result =
+  let result (type r) ~errno ~stub_name fmt fn (result : r typ) =
     begin
       fprintf fmt "@[static@ value@ result_%s@;@[(struct@ job_%s@ *j)@]@]@;@[<2>{@\n"
         stub_name stub_name;
@@ -407,54 +439,28 @@ struct
           fprintf fmt "@[Store_field(rv,@ 1,@ ctypes_copy_sint(j->error_status));@]@\n";
           fprintf fmt "@[Store_field(rv,@ 0,@ ";
       in
-          fprintf fmt "%a);@]@\n"
-            (fun fmt ty ->
-               Cstubs_emit_c.ceff fmt
-                 (Generate_C.inj ty
-                    (`Local ("j->result", Cstubs_c_language.(Ty ty)))))
-            result;
+      fprintf fmt "%a);@]@\n"
+        (let f (type r) fmt : r typ -> _ = function
+             Void ->
+             Cstubs_emit_c.ceff fmt Generate_C.val_unit
+           | ty ->
+             Cstubs_emit_c.ceff fmt
+               (Generate_C.inj ty
+                  (`Local ("j->result", Cstubs_c_language.(Ty ty))))
+         in f
+        )
+        result;
       fprintf fmt "@[lwt_unix_free_job(&j->job)@];@\n";
       fprintf fmt "@[CAMLreturn@ (rv)@];@]@\n";
       fprintf fmt "}@\n";
     end
 
-  let rec camlxParam fmt args =
-    match args with
-      [] -> ()
-    | x1 :: [] ->
-      fprintf fmt "@[CAMLxparam1 (%s)@];@\n" x1
-    | x1 :: x2 :: [] ->
-      fprintf fmt "@[CAMLxparam2 (%s, %s)@];@\n" x1 x2
-    | x1 :: x2 :: x3 :: [] ->
-      fprintf fmt "@[CAMLxparam3 (%s, %s, %s)@];@\n" x1 x2 x3
-    | x1 :: x2 :: x3 :: x4 :: [] ->
-      fprintf fmt "@[CAMLxparam4 (%s, %s, %s, %s)@];@\n" x1 x2 x3 x4
-    | x1 :: x2 :: x3 :: x4 :: x5 :: rest ->
-      fprintf fmt "@[CAMLxparam5 (%s, %s, %s, %s, %s)@];@\n" x1 x2 x3 x4 x5;
-      camlxParam fmt rest
-
-  let camlParam fmt args =
-    match args with
-      [] ->
-      fprintf fmt "@[CAMLparam0 ()@];@\n"
-    | x1 :: [] ->
-      fprintf fmt "@[CAMLparam1 (%s)@];@\n" x1
-    | x1 :: x2 :: [] ->
-      fprintf fmt "@[CAMLparam2 (%s, %s)@];@\n" x1 x2
-    | x1 :: x2 :: x3 :: [] ->
-      fprintf fmt "@[CAMLparam3 (%s, %s, %s)@];@\n" x1 x2 x3
-    | x1 :: x2 :: x3 :: x4 :: [] ->
-      fprintf fmt "@[CAMLparam4 (%s, %s, %s, %s)@];@\n" x1 x2 x3 x4
-    | x1 :: x2 :: x3 :: x4 :: x5 :: rest ->
-      fprintf fmt "@[CAMLparam5 (%s, %s, %s, %s, %s)@];@\n" x1 x2 x3 x4 x5;
-      camlxParam fmt rest
-
   let stub ~errno ~stub_name fmt fn args =
     begin
       fprintf fmt "@[value@ %s@;@[(%s)@]@]@;@[<2>{@\n"
         stub_name
         (String.concat ", " (List.map (fun (_, x) -> "value "^ x) args));
-      camlParam fmt (List.map snd args);
+      Cstubs_emit_c.camlParam fmt (List.map snd args);
 
       fprintf fmt "@[LWT_UNIX_INIT_JOB(job,@ %s,@ 0)@];@\n"
         stub_name;
@@ -514,6 +520,10 @@ struct
     end
 end
 
-let fn ~concurrency ~errno = match concurrency with
-    `Sequential -> fn ~errno
-  | `Lwt_jobs -> Lwt.fn ~errno
+let fn ~concurrency ~errno ~cname ~stub_name fmt f = match concurrency with
+  | `Lwt_preemptive | `Unlocked | `Lwt_jobs when has_ocaml_argument f ->
+    raise (Unsupported "Unsupported argument type when releasing runtime lock")
+  | `Lwt_preemptive | `Unlocked -> fn ~concurrency:`Unlocked ~errno ~cname ~stub_name fmt f
+  | `Sequential -> fn ~concurrency:`Sequential ~errno ~cname ~stub_name fmt f
+  | `Lwt_jobs -> Lwt.fn ~errno ~cname ~stub_name fmt f
+
