@@ -46,32 +46,36 @@ type 'a bigarray_class = 'a Ctypes_static.bigarray_class
 
 val genarray :
   < element: 'a;
+    layout: 'l;
     ba_repr: 'b;
-    bigarray: ('a, 'b, Bigarray.c_layout) Bigarray.Genarray.t;
+    bigarray: ('a, 'b, 'l) Bigarray.Genarray.t;
     carray: 'a carray;
     dims: int array > bigarray_class
 (** The class of {!Bigarray.Genarray.t} values *)
 
 val array1 :
   < element: 'a;
+    layout: 'l;
     ba_repr: 'b;
-    bigarray: ('a, 'b, Bigarray.c_layout) Bigarray.Array1.t;
+    bigarray: ('a, 'b, 'l) Bigarray.Array1.t;
     carray: 'a carray;
     dims: int > bigarray_class
 (** The class of {!Bigarray.Array1.t} values *)
 
 val array2 :
   < element: 'a;
+    layout: 'l;
     ba_repr: 'b;
-    bigarray: ('a, 'b, Bigarray.c_layout) Bigarray.Array2.t;
+    bigarray: ('a, 'b, 'l) Bigarray.Array2.t;
     carray: 'a carray carray;
     dims: int * int > bigarray_class
 (** The class of {!Bigarray.Array2.t} values *)
 
 val array3 :
   < element: 'a;
+    layout: 'l;
     ba_repr: 'b;
-    bigarray: ('a, 'b, Bigarray.c_layout) Bigarray.Array3.t;
+    bigarray: ('a, 'b, 'l) Bigarray.Array3.t;
     carray: 'a carray carray carray;
     dims: int * int * int > bigarray_class
 (** The class of {!Bigarray.Array3.t} values *)
@@ -200,6 +204,9 @@ val ptr_compare : 'a ptr -> 'a ptr -> int
     negative if [i] is less than [j], positive if [i] is greater than [j], and
     zero if [i] and [j] are equal. *)
 
+val is_null : 'a ptr -> bool
+(** [is_null p] is true when [p] is a null pointer. *)
+
 val reference_type : 'a ptr -> 'a typ
 (** Retrieve the reference type of a pointer. *)
 
@@ -228,7 +235,7 @@ val ocaml_string_start : string -> string ocaml
 (** [ocaml_string_start s] allows to pass a pointer to the contents of an OCaml
     string directly to a C function. *)
 
-val ocaml_bytes_start : Bytes.t -> Bytes.t ocaml
+val ocaml_bytes_start : bytes -> bytes ocaml
 (** [ocaml_bytes_start s] allows to pass a pointer to the contents of an OCaml
     byte array directly to a C function. *)
 
@@ -268,6 +275,10 @@ sig
   (** [unsafe_set a n v] behaves like [set a n v] except that the check that
       [n] between [0] and [(CArray.length a - 1)] is not performed. *)
 
+  val of_string : string -> char t
+  (** [of_string s] builds an array of the same length as [s], and writes
+      the elements of [s] to the corresponding elements of the array. *)
+
   val of_list : 'a typ -> 'a list -> 'a t
   (** [of_list t l] builds an array of type [t] of the same length as [l], and
       writes the elements of [l] to the corresponding elements of the array. *)
@@ -277,6 +288,30 @@ sig
       element of the list is the result of reading the corresponding element of
       [a]. *)
 
+  val iter : ('a -> unit) -> 'a t -> unit
+  (** [iter f a] is analogous to [Array.iter f a]: it applies [f] in turn to
+      all the elements of [a]. *)
+
+  val map : 'b typ -> ('a -> 'b) -> 'a t -> 'b t
+  (** [map t f a] is analogous to [Array.map f a]: it creates a new array with
+      element type [t] whose elements are obtained by applying [f] to the
+      elements of [a]. *)
+
+  val mapi : 'b typ -> (int -> 'a -> 'b) -> 'a t -> 'b t
+  (** [mapi] behaves like {!Array.mapi}, except that it also passes the
+      index of each element as the first argument to [f] and the element
+      itself as the second argument. *)
+
+  val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
+  (** [CArray.fold_left (@) x a] computes 
+         [(((x @ a.(0)) @ a.(1)) ...) @ a.(n-1)]
+       where [n] is the length of the array [a]. *)
+
+  val fold_right : ('b -> 'a -> 'a) -> 'b t -> 'a -> 'a
+  (** [CArray.fold_right f a x] computes
+         [a.(0) @ (a.(1) @ ( ... (a.(n-1) @ x) ...))]
+       where [n] is the length of the array [a]. *)
+
   val length : 'a t -> int
   (** Return the number of elements of the given array. *)
 
@@ -293,14 +328,25 @@ sig
       used to initialise every element of the array.  The argument [?finalise],
       if present, will be called just before the memory is freed. *)
 
+  val copy : 'a t -> 'a t
+  (** [copy a] creates a fresh array with the same elements as [a]. *)
+
+  val sub : 'a t -> pos:int -> length:int -> 'a t
+  (** [sub a ~pos ~length] creates a fresh array of length [length] containing
+      the elements [a.(pos)] to [a.(pos + length - 1)] of [a].
+
+      Raise [Invalid_argument "CArray.sub"] if [pos] and [length] do not
+      designate a valid subarray of [a]. *)
+
   val element_type : 'a t -> 'a typ
-(** Retrieve the element type of an array. *)
+  (** Retrieve the element type of an array. *)
 end
 (** Operations on C arrays. *)
 
 (** {4 Bigarray values} *)
 
 val bigarray_start : < element: 'a;
+                       layout: 'l;
                        ba_repr: _;
                        bigarray: 'b;
                        carray: _;
@@ -308,16 +354,29 @@ val bigarray_start : < element: 'a;
 (** Return the address of the first element of the given Bigarray value. *)
 
 val bigarray_of_ptr : < element: 'a;
+                        layout: Bigarray.c_layout;
                         ba_repr: 'f;
                         bigarray: 'b;
                         carray: _;
                         dims: 'i > bigarray_class ->
     'i -> ('a, 'f) Bigarray.kind -> 'a ptr -> 'b
-(** [bigarray_of_ptr c dims k p] converts the C pointer [p] to a bigarray
-    value.  No copy is made; the bigarray references the memory pointed to by
-    [p]. *)
+(** [bigarray_of_ptr c dims k p] converts the C pointer [p] to a C-layout
+    bigarray value.  No copy is made; the bigarray references the memory
+    pointed to by [p]. *)
+
+val fortran_bigarray_of_ptr : < element: 'a;
+                                layout: Bigarray.fortran_layout;
+                                ba_repr: 'f;
+                                bigarray: 'b;
+                                carray: _;
+                                dims: 'i > bigarray_class ->
+    'i -> ('a, 'f) Bigarray.kind -> 'a ptr -> 'b
+(** [fortran_bigarray_of_ptr c dims k p] converts the C pointer [p] to a
+    Fortran-layout bigarray value.  No copy is made; the bigarray references
+    the memory pointed to by [p]. *)
 
 val array_of_bigarray : < element: _;
+                          layout: Bigarray.c_layout;
                           ba_repr: _;
                           bigarray: 'b;
                           carray: 'c;
@@ -329,13 +388,15 @@ val array_of_bigarray : < element: _;
 (** Convert a Bigarray value to a C array. *)
 
 val bigarray_of_array : < element: 'a;
+                          layout: Bigarray.c_layout;
                           ba_repr: 'f;
                           bigarray: 'b;
                           carray: 'c carray;
                           dims: 'i > bigarray_class ->
     ('a, 'f) Bigarray.kind -> 'c carray -> 'b
-(** [bigarray_of_array c k a] converts the {!CArray.t} value [c] to a bigarray
-    value.  No copy is made; the result occupies the same memory as [c]. *)
+(** [bigarray_of_array c k a] converts the {!CArray.t} value [a] to a
+    C-layout bigarray value.  No copy is made; the result occupies the
+    same memory as [a]. *)
 
 (** {3 Struct and union values} *)
 
@@ -414,6 +475,97 @@ val coerce_fn : 'a fn -> 'b fn -> 'a -> 'b
     ctypes may both add new types of coercion and restrict the existing
     coercions. *)
 
+
+(** {2 binding interfaces}.
+*)
+
+(** Foreign function binding interface.
+
+    The {!Foreign} and {!Cstubs} modules provide concrete implementations. *)
+module type FOREIGN =
+sig
+  type 'a fn
+  type 'a return
+  val (@->) : 'a typ -> 'b fn -> ('a -> 'b) fn
+  val returning : 'a typ -> 'a return fn
+
+  type 'a result
+  val foreign : string -> ('a -> 'b) fn -> ('a -> 'b) result
+  val foreign_value : string -> 'a typ -> 'a ptr result
+end
+
+(** Foreign types binding interface.
+
+    The {!Cstubs} module builds concrete implementations. *)
+module type TYPE =
+sig
+  include Ctypes_types.TYPE
+
+  type 'a const
+  val constant : string -> 'a typ -> 'a const
+  (** [constant name typ] retrieves the value of the compile-time constant
+      [name] of type [typ].  It can be used to retrieve enum constants,
+      #defined values and other integer constant expressions.
+
+      The type [typ] must be either an integer type such as [bool], [char],
+      [int], [uint8], etc., or a view (or perhaps multiple views) where the
+      underlying type is an integer type.
+
+      When the value of the constant cannot be represented in the type there
+      will typically be a diagnostic from either the C compiler or the OCaml
+      compiler.  For example, gcc will say
+
+         warning: overflow in implicit constant conversion *)
+
+  val enum : string -> ?typedef:bool ->
+    ?unexpected:(int64 -> 'a) -> ('a * int64 const) list -> 'a typ
+  (** [enum name ?unexpected alist] builds a type representation for the
+      enum named [name].  The size and alignment are retrieved so that the
+      resulting type can be used everywhere an integer type can be used: as
+      an array element or struct member, as an argument or return value,
+      etc.
+
+      The value [alist] is an association list of OCaml values and values
+      retrieved by the [constant] function.  For example, to expose the enum
+
+        enum letters \{ A, B, C = 10, D \}; 
+
+      you might first retrieve the values of the enumeration constants:
+
+      {[
+        let a = constant "A" int64_t
+        and b = constant "B" int64_t
+        and c = constant "C" int64_t
+        and d = constant "D" int64_t
+      ]}
+
+      and then build the enumeration type
+
+      {[
+        let letters = enum "letters" [
+           `A, a;
+           `B, b;
+           `C, c;
+           `D, d;
+        ] ~unexpected:(fun i -> `E i)
+      ]}
+
+      The [unexpected] function specifies the value to return in the case
+      that some unexpected value is encountered -- for example, if a
+      function with the return type 'enum letters' actually returns the
+      value [-1].
+
+      The optional flag [typedef] specifies whether the first argument,
+      [name], indicates an tag or an alias.  If [typedef] is [false] (the
+      default) then [name] is treated as an enumeration tag:
+
+        [enum letters { ... }]
+
+      If [typedef] is [true] then [name] is instead treated as an alias:
+
+        [typedef enum { ... } letters] *)
+end
+
 (** {2:roots Registration of OCaml values as roots} *)
 module Root :
 sig
